"""
Play elevator music in the background while your script runs.
"""
from script_background_music.play import play_music_in_background, play_random_music_in_background, \
    play_song_in_background
from script_background_music.context import BackgroundMusicContext



"""
Use the elevator music as a context
"""
from typing import Callable, Union

from script_background_music.play import play_random_music_in_background, PlayThread


class BackgroundMusicContext(object):
    """
    Run background music in a given context

    <b>WARNING: Not verified to work on macOS! Should work there as well, since also creates a subprocess under the hood.</b>
    """
    __thread: Union[PlayThread, None] = None

    def __init__(self, player_callback: Callable[[], PlayThread] = play_random_music_in_background):
        """
        Create new background music context

        :param player_callback: Callback to run the music, defaults to<i>script_background_music.play.play_random_music_in_background</i>
        """
        self.__thread = player_callback()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.__thread.kill_children()
        self.__thread.terminate()



"""
Infrastructure to facilitate playing songs
"""
import logging
from multiprocessing import Process
from typing import Union
import psutil
from script_background_music.songs import get_random_song
from script_background_music.vendor.playsound import get_play_sound


class PlayThread(Process):
    """
    Thread to be used for playing songs
    """
    def kill_children(self):
        """
        Ensures all player child threads are terminated properly
        """
        process = psutil.Process(self.pid)
        for child in process.children(recursive=True):
            child.kill()


def __music(song_title):
    try:
        while True:
            get_play_sound()(song_title, block=True)
    except:
        logging.debug("Could not start music, exiting.")


def play_music_in_background(song_file: Union[str, None] = None) -> PlayThread:
    """
    Play music in background if no song is provided, a random will be picked

    :param song_file: Song to play, set to None or omit to use a random one
    :return: Underlying thread for playing music
    """
    song_to_play = get_random_song() if song_file is None else song_file
    thread = PlayThread(target=__music, args=(song_to_play,))
    thread.start()
    return thread


def play_random_music_in_background() -> PlayThread:
    """
    Play random music from the available songs in the background

    :return: Underlying thread for playing music
    """
    return play_music_in_background(None)


def play_song_in_background(file_name: str) -> PlayThread:
    """
    Play song from given file in background

    :param file_name: Filename of song to play in background
    :return: Underlying thread for playing music
    """
    return play_music_in_background(file_name)



from setuptools import setup, find_packages
from os import environ

with open("README.md", "r") as readme_file:
    readme = "".join(readme_file.readlines()) \
        .replace(
        ".github/images/elevator.png",
        "https://raw.githubusercontent.com/timo-reymann/python-script-background-music/main/.github/images/elevator.png"
    )

setup(
    name='script-background-music',
    version=environ.get("VERSION", "snapshot"),
    author="Timo Reymann <mail@timo-reymann.de>",
    url="https://github.com/timo-reymann/python-script-background-music",
    include_package_data=True,
    packages=find_packages(),
    requires=[],
    python_requires='>3.8.0',
    description="Play elevator music in the background while your script runs.",
    long_description=readme,
    long_description_content_type='text/markdown'
)
